@@ -1,3 +1,24 @@
+/* Clearlooks theme engine
+ * Copyright (C) 2005 Richard Stellingwerff.
+ * Copyright (C) 2007 Benjamin Berg <benjamin@sipsolutions.net>.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ */
+
 #include <gtk/gtk.h>
 #include <cairo.h>
 #include <math.h>
@@ -97,10 +118,29 @@ clearlooks_style_draw_flat_box (DRAW_ARGS)
 
 		cairo_destroy (cr);
 	}
+	else if (DETAIL ("tooltip"))
+	{
+		WidgetParameters params;
+		ClearlooksStyle  *clearlooks_style;
+		ClearlooksColors *colors;
+		cairo_t          *cr;
+
+		CHECK_ARGS
+		SANITIZE_SIZE
+
+		clearlooks_style = CLEARLOOKS_STYLE (style);
+		clearlooks_set_widget_parameters (widget, style, state_type, &params);
+		colors = &clearlooks_style->colors;
+		cr = ge_gdk_drawable_to_cairo (window, area);
+
+		STYLE_FUNCTION (draw_tooltip) (cr, colors, &params, x, y, width, height);
+
+		cairo_destroy (cr);
+	}
 	else if ((CLEARLOOKS_STYLE (style)->style == CL_STYLE_GLOSSY || CLEARLOOKS_STYLE (style)->style == CL_STYLE_GUMMY) &&
 	         ((DETAIL("checkbutton") || DETAIL("radiobutton")) && state_type == GTK_STATE_PRELIGHT))
 	{
-		/* XXX: Don't draw any check/radiobutton bg in GLOSSY mode. */
+		/* XXX: Don't draw any check/radiobutton bg in GLOSSY or GUMMY mode. */
 	}
 	else
 	{
@@ -214,6 +254,7 @@ clearlooks_style_draw_box_gap (DRAW_ARGS,
 	{
 		WidgetParameters params;
 		FrameParameters  frame;
+		int current_page, num_pages;
 		
 		frame.shadow    = shadow_type;
 		frame.gap_side  = gap_side;
@@ -229,8 +270,8 @@ clearlooks_style_draw_box_gap (DRAW_ARGS,
 		Last Active tab could be buggied if the tabs cover more space then the notebook width
 		***
 		*/
-		int current_page = gtk_notebook_get_current_page ((GtkNotebook *) widget);
-		int num_pages = gtk_notebook_get_n_pages ((GtkNotebook *) widget);
+		current_page = gtk_notebook_get_current_page ((GtkNotebook *) widget);
+		num_pages = gtk_notebook_get_n_pages ((GtkNotebook *) widget);
 
 		if (frame.gap_side == CL_GAP_TOP) {
 			if ((current_page == 0) && (num_pages - 1 == 0)) {
@@ -394,8 +435,14 @@ clearlooks_style_draw_handle (DRAW_ARGS, GtkOrientation orientation)
 		/* Is this ever true? -Daniel */
 		if (GE_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
+			ToolbarParameters toolbar;
+
+			clearlooks_set_toolbar_parameters (&toolbar, widget, window, x, y);
+
+			toolbar.style = clearlooks_style->toolbarstyle;
+
 			cairo_save (cr);
-			STYLE_FUNCTION(draw_toolbar) (cr, colors, &params, x, y, width, height);
+			STYLE_FUNCTION(draw_toolbar) (cr, colors, &params, &toolbar, x, y, width, height);
 			cairo_restore (cr);
 		}
 		
@@ -426,8 +473,14 @@ clearlooks_style_draw_handle (DRAW_ARGS, GtkOrientation orientation)
 		/* Is this ever true? -Daniel */
 		if (GE_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
+			ToolbarParameters toolbar;
+
+			clearlooks_set_toolbar_parameters (&toolbar, widget, window, x, y);
+
+			toolbar.style = clearlooks_style->toolbarstyle;
+
 			cairo_save (cr);
-			STYLE_FUNCTION(draw_toolbar) (cr, colors, &params, x, y, width, height);
+			STYLE_FUNCTION(draw_toolbar) (cr, colors, &params, &toolbar, x, y, width, height);
 			cairo_restore (cr);
 		}
 		
@@ -541,13 +594,6 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 			params.active = TRUE;
 		
-		/* Fix some firefox crap. */
-		if (GE_IS_BUTTON (widget) && GE_IS_FIXED (widget->parent) && widget->allocation.x == -1 &&  widget->allocation.y == -1)
-		{
-			gtk_style_apply_default_background (widget->parent->style, window, TRUE, GTK_STATE_NORMAL,
-			                                    area, x, y, width, height);
-		}
-		
 		STYLE_FUNCTION(draw_button) (cr, &clearlooks_style->colors, &params,
 		                             x, y, width, height);
 	}
@@ -614,6 +660,8 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		params.corners    = CR_CORNER_NONE;
 		
 		slider.lower = DETAIL ("trough-lower");
+		slider.fill_level = DETAIL ("trough-fill-level") || DETAIL ("trough-fill-level-full");
+
 		slider.horizontal = (GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL);
 		
 		STYLE_FUNCTION(draw_scale_trough) (cr, &clearlooks_style->colors,
@@ -810,9 +858,17 @@ clearlooks_style_draw_box (DRAW_ARGS)
 	}
 	else if (DETAIL ("toolbar") || DETAIL ("handlebox_bin") || DETAIL ("dockitem_bin"))
 	{
+		WidgetParameters  params;
+		ToolbarParameters toolbar;
+
+		clearlooks_set_widget_parameters (widget, style, state_type, &params);
+		clearlooks_set_toolbar_parameters (&toolbar, widget, window, x, y);
+
+		toolbar.style = clearlooks_style->toolbarstyle;
+
 		/* Only draw the shadows on horizontal toolbars */
 		if (shadow_type != GTK_SHADOW_NONE && height < 2*width )
-			STYLE_FUNCTION(draw_toolbar) (cr, colors, NULL, x, y, width, height);
+			STYLE_FUNCTION(draw_toolbar) (cr, colors, &params, &toolbar, x, y, width, height);
 	}
 	else if (DETAIL ("trough"))
 	{
@@ -856,6 +912,8 @@ clearlooks_style_draw_slider (DRAW_ARGS, GtkOrientation orientation)
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		
 		slider.horizontal = (orientation == GTK_ORIENTATION_HORIZONTAL);
+		slider.lower = FALSE;
+		slider.fill_level = FALSE;
 		
 		if (clearlooks_style->style == CL_STYLE_GLOSSY) /* XXX! */
 			params.corners = CR_CORNER_ALL;
@@ -876,18 +934,21 @@ clearlooks_style_draw_slider (DRAW_ARGS, GtkOrientation orientation)
 		scrollbar.horizontal = (orientation == GTK_ORIENTATION_HORIZONTAL);
 		scrollbar.junction   = clearlooks_scrollbar_get_junction (widget);
 
-		if (clearlooks_style->style == CL_STYLE_GLOSSY || clearlooks_style->style == CL_STYLE_GUMMY)
-		{
+		if (clearlooks_style->colorize_scrollbar) {
 			scrollbar.color = colors->spot[1];
 			scrollbar.has_color = TRUE;
 		}
 
-		/* Set scrollbar color also for Glossy */
+		/* Set scrollbar color */
 		if (clearlooks_style->has_scrollbar_color)
 		{
 			ge_gdk_color_to_cairo (&clearlooks_style->scrollbar_color, &scrollbar.color);
 			scrollbar.has_color = TRUE;
 		}
+
+		if ((clearlooks_style->style == CL_STYLE_GLOSSY || clearlooks_style->style == CL_STYLE_GUMMY)
+			&& !scrollbar.has_color)
+			scrollbar.color = colors->bg[0];
 		
 		STYLE_FUNCTION(draw_scrollbar_slider) (cr, colors, &params, &scrollbar,
 		                                       x, y, width, height);
@@ -908,11 +969,11 @@ clearlooks_style_draw_option (DRAW_ARGS)
 	const ClearlooksColors *colors;
 	WidgetParameters params;
 	CheckboxParameters checkbox;
+	cairo_t *cr;
 	
 	CHECK_ARGS
 	SANITIZE_SIZE
 
-	cairo_t *cr;
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	colors = &clearlooks_style->colors;
 	
@@ -1181,7 +1242,9 @@ clearlooks_style_init_from_rc (GtkStyle * style,
 	clearlooks_style->style		= CLEARLOOKS_RC_STYLE (rc_style)->style;
 	
 	clearlooks_style->menubarstyle      = CLEARLOOKS_RC_STYLE (rc_style)->menubarstyle;
+	clearlooks_style->toolbarstyle      = CLEARLOOKS_RC_STYLE (rc_style)->toolbarstyle;
 	clearlooks_style->has_scrollbar_color = CLEARLOOKS_RC_STYLE (rc_style)->flags & CL_FLAG_SCROLLBAR_COLOR;
+	clearlooks_style->colorize_scrollbar = CLEARLOOKS_RC_STYLE (rc_style)->colorize_scrollbar;
 	clearlooks_style->animation         = CLEARLOOKS_RC_STYLE (rc_style)->animation;
 	clearlooks_style->radius            = CLAMP (CLEARLOOKS_RC_STYLE (rc_style)->radius, 0.0, 10.0);
 
@@ -1226,20 +1289,6 @@ clearlooks_style_realize (GtkStyle * style)
 	}
 }
 
-static void
-gdk_cairo_set_source_color_alpha (cairo_t  *cr,
-                GdkColor *color, float alpha)
-{
-  g_return_if_fail (cr != NULL);
-  g_return_if_fail (color != NULL);
-
-  cairo_set_source_rgba (cr,
-            color->red / 65535.,
-            color->green / 65535.,
-            color->blue / 65535.,
-	        alpha);
-}
-
 static void
 clearlooks_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
             GdkRectangle *area, GtkWidget *widget, const gchar *detail,
@@ -1279,8 +1328,8 @@ clearlooks_style_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType st
 	else if (detail && !strcmp (detail, "colorwheel_dark"))
 		cairo_set_source_rgb (cr, 1., 1., 1.);
 	else
-		gdk_cairo_set_source_color_alpha (cr, &style->fg[state_type],
-						  0.7);
+		ge_cairo_set_gdk_color_with_alpha (cr, &style->fg[state_type],
+						       0.7);
 
 	cairo_set_line_width (cr, line_width);
 
@@ -1336,8 +1385,10 @@ clearlooks_style_copy (GtkStyle * style, GtkStyle * src)
 	
 	cl_style->colors              = cl_src->colors;
 	cl_style->menubarstyle        = cl_src->menubarstyle;
+	cl_style->toolbarstyle        = cl_src->toolbarstyle;
 	cl_style->scrollbar_color     = cl_src->scrollbar_color;
 	cl_style->has_scrollbar_color = cl_src->has_scrollbar_color;
+	cl_style->colorize_scrollbar  = cl_src->colorize_scrollbar;
 	cl_style->animation           = cl_src->animation;
 	cl_style->radius              = cl_src->radius;
 	cl_style->style               = cl_src->style;
@@ -1409,43 +1460,43 @@ clearlooks_style_draw_layout (GtkStyle * style,
 	     GtkWidget * widget,
 	     const gchar * detail, gint x, gint y, PangoLayout * layout)
 {
-  GdkGC *gc;
+	GdkGC *gc;
 
-  g_return_if_fail (GTK_IS_STYLE (style));
-  g_return_if_fail (window != NULL);
+	g_return_if_fail (GTK_IS_STYLE (style));
+	g_return_if_fail (window != NULL);
 
-  gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
+	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
 
-  if (area)
-    gdk_gc_set_clip_rectangle (gc, area);
+	if (area)
+		gdk_gc_set_clip_rectangle (gc, area);
 
-  if (state_type == GTK_STATE_INSENSITIVE) {
-    ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
-    ClearlooksColors *colors = &clearlooks_style->colors;
+	if (state_type == GTK_STATE_INSENSITIVE) {
+		ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
+		ClearlooksColors *colors = &clearlooks_style->colors;
 
-    WidgetParameters params;
+		WidgetParameters params;
+		GdkColor etched;
+		CairoColor temp;
 
-    clearlooks_set_widget_parameters (widget, style, state_type, &params);
+		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
-    GdkColor etched;
-    CairoColor temp;
+		if (GTK_WIDGET_NO_WINDOW (widget))
+			ge_shade_color (&params.parentbg, 1.2, &temp);
+		else
+			ge_shade_color (&colors->bg[widget->state], 1.2, &temp);
+		
+		etched.red = (int) (temp.r * 65535);
+		etched.green = (int) (temp.g * 65535);
+		etched.blue = (int) (temp.b * 65535);
 
-    if (GTK_WIDGET_NO_WINDOW (widget))
-      ge_shade_color (&params.parentbg, 1.2, &temp);
-    else
-      ge_shade_color (&colors->bg[widget->state], 1.2, &temp);
-    etched.red = (int) (temp.r * 65535);
-    etched.green = (int) (temp.g * 65535);
-    etched.blue = (int) (temp.b * 65535);
+		gdk_draw_layout_with_colors (window, style->text_gc[state_type], x + 1, y + 1, layout, &etched, NULL);
+		gdk_draw_layout (window, style->text_gc[state_type], x, y, layout);
+	}
+	else
+		gdk_draw_layout (window, gc, x, y, layout);
 
-    gdk_draw_layout_with_colors (window, style->text_gc[state_type], x + 1, y + 1, layout, &etched, NULL);
-    gdk_draw_layout (window, style->text_gc[state_type], x, y, layout);
-  }
-  else
-    gdk_draw_layout (window, gc, x, y, layout);
-
-  if (area)
-    gdk_gc_set_clip_rectangle (gc, NULL);
+	if (area)
+		gdk_gc_set_clip_rectangle (gc, NULL);
 }
 
 static GdkPixbuf *
